import type { BindingNormalizedOptions } from '../binding';
import type { SourcemapIgnoreListOption, SourcemapPathTransformOption } from '../types/misc';
import type { ChunkFileNamesFunction, GlobalsFunction, OutputOptions } from './output-options';
export type InternalModuleFormat = 'es' | 'cjs' | 'iife' | 'umd' | 'app';
export interface NormalizedOutputOptions {
    name: string | undefined;
    file: string | undefined;
    dir: string | undefined;
    entryFileNames: string | ChunkFileNamesFunction;
    chunkFileNames: string | ChunkFileNamesFunction;
    assetFileNames: string;
    format: InternalModuleFormat;
    exports: NonNullable<OutputOptions['exports']>;
    sourcemap: boolean | 'inline' | 'hidden';
    cssEntryFileNames: string | ChunkFileNamesFunction;
    cssChunkFileNames: string | ChunkFileNamesFunction;
    inlineDynamicImports: boolean;
    externalLiveBindings: boolean;
    banner: OutputOptions['banner'];
    footer: OutputOptions['footer'];
    intro: OutputOptions['intro'];
    outro: OutputOptions['outro'];
    esModule: boolean | 'if-default-prop';
    extend: boolean;
    globals: Record<string, string> | GlobalsFunction;
    hashCharacters: 'base64' | 'base36' | 'hex';
    sourcemapDebugIds: boolean;
    sourcemapIgnoreList: SourcemapIgnoreListOption | undefined;
    sourcemapPathTransform: SourcemapPathTransformOption | undefined;
    minify: boolean;
    comments: 'none' | 'preserve-legal';
    polyfillRequire: boolean;
}
type UnsupportedFnRet = () => never;
export declare class NormalizedOutputOptionsImpl implements NormalizedOutputOptions {
    inner: BindingNormalizedOptions;
    constructor(inner: BindingNormalizedOptions);
    get dir(): string | undefined;
    get entryFileNames(): string | UnsupportedFnRet;
    get chunkFileNames(): string | UnsupportedFnRet;
    get assetFileNames(): string;
    get format(): 'es' | 'cjs' | 'app' | 'iife' | 'umd';
    get exports(): 'default' | 'named' | 'none' | 'auto';
    get sourcemap(): boolean | 'inline' | 'hidden';
    get cssEntryFileNames(): string | UnsupportedFnRet;
    get cssChunkFileNames(): string | UnsupportedFnRet;
    get shimMissingExports(): boolean;
    get name(): string | undefined;
    get file(): string | undefined;
    get inlineDynamicImports(): boolean;
    get externalLiveBindings(): boolean;
    get banner(): string | UnsupportedFnRet | undefined;
    get footer(): string | UnsupportedFnRet | undefined;
    get intro(): string | UnsupportedFnRet | undefined;
    get outro(): string | UnsupportedFnRet | undefined;
    get esModule(): boolean | 'if-default-prop';
    get extend(): boolean;
    get globals(): Record<string, string> | UnsupportedFnRet;
    get hashCharacters(): 'base64' | 'base36' | 'hex';
    get sourcemapDebugIds(): boolean;
    get sourcemapIgnoreList(): UnsupportedFnRet | undefined;
    get sourcemapPathTransform(): UnsupportedFnRet | undefined;
    get minify(): boolean;
    get comments(): 'none' | 'preserve-legal';
    get polyfillRequire(): boolean;
}
export {};
